'use strict';

const postcss = require('postcss');
const parser = require('postcss-selector-parser');
const parse5 = require('parse5');
const shared = require('@tailwindcss-mangle/shared');
const t = require('@babel/types');
const core = require('@babel/core');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

function _interopNamespaceCompat(e) {
  if (e && typeof e === 'object' && 'default' in e) return e;
  const n = Object.create(null);
  if (e) {
    for (const k in e) {
      n[k] = e[k];
    }
  }
  n.default = e;
  return n;
}

const postcss__default = /*#__PURE__*/_interopDefaultCompat(postcss);
const parser__default = /*#__PURE__*/_interopDefaultCompat(parser);
const t__namespace = /*#__PURE__*/_interopNamespaceCompat(t);

function isObject(value) {
  return value !== null && typeof value === "object";
}
function _defu(baseObject, defaults, namespace = ".", merger) {
  if (!isObject(defaults)) {
    return _defu(baseObject, {}, namespace, merger);
  }
  const object = Object.assign({}, defaults);
  for (const key in baseObject) {
    if (key === "__proto__" || key === "constructor") {
      continue;
    }
    const value = baseObject[key];
    if (value === null || value === void 0) {
      continue;
    }
    if (merger && merger(object, key, value, namespace)) {
      continue;
    }
    if (Array.isArray(value) && Array.isArray(object[key])) {
      object[key] = [...value, ...object[key]];
    } else if (isObject(value) && isObject(object[key])) {
      object[key] = _defu(
        value,
        object[key],
        (namespace ? `${namespace}.` : "") + key.toString(),
        merger
      );
    } else {
      object[key] = value;
    }
  }
  return object;
}
function createDefu(merger) {
  return (...arguments_) => (
    // eslint-disable-next-line unicorn/no-array-reduce
    arguments_.reduce((p, c) => _defu(p, c, "", merger), {})
  );
}
const defu = createDefu();

const postcssPlugin = "postcss-mangle-tailwindcss-plugin";
function isVueScoped(s) {
  if (s.parent) {
    const index = s.parent.nodes.indexOf(s);
    if (index > -1) {
      const nextNode = s.parent.nodes[index + 1];
      if (nextNode && nextNode.type === "attribute" && nextNode.attribute.includes("data-v-")) {
        return true;
      }
    }
  }
  return false;
}
const postcssMangleTailwindcssPlugin = (options) => {
  const { ignoreVueScoped } = defu(options, {
    ignoreVueScoped: true
  });
  if (options?.scene === "loader") {
    let set = /* @__PURE__ */ new Set();
    if (options && options.runtimeSet) {
      set = options.runtimeSet;
    }
    return {
      postcssPlugin,
      Rule(rule) {
        rule.selector = parser__default((selectors) => {
          selectors.walkClasses((s) => {
            if (s.value) {
              const existed = set.has(s.value);
              if (existed) {
                if (ignoreVueScoped && isVueScoped(s)) {
                  return;
                }
                s.value = options.classGenerator.generateClassName(s.value).name;
              }
            }
          });
        }).processSync(rule.selector);
      }
    };
  } else {
    let newClassMap = {};
    if (options && options.classGenerator) {
      newClassMap = options.classGenerator.newClassMap;
    }
    return {
      postcssPlugin,
      Rule(rule) {
        rule.selector = parser__default((selectors) => {
          selectors.walkClasses((s) => {
            if (s.value) {
              const hit = newClassMap[s.value];
              const existed = Boolean(hit);
              if (existed) {
                if (ignoreVueScoped && isVueScoped(s)) {
                  return;
                }
                s.value = hit.name;
              }
            }
          });
        }).processSync(rule.selector);
      }
    };
  }
};
postcssMangleTailwindcssPlugin.postcss = true;

function cssHandler(rawSource, options) {
  const acceptedPlugins = [postcssMangleTailwindcssPlugin(options)];
  return postcss__default(acceptedPlugins).process(rawSource).css;
}

({
    HTML: parse5.html.NS.HTML,
    XML: parse5.html.NS.XML,
    MATHML: parse5.html.NS.MATHML,
    SVG: parse5.html.NS.SVG,
    XLINK: parse5.html.NS.XLINK,
    XMLNS: parse5.html.NS.XMLNS
});

/**
 * Determines if a given node is a document or not
 * @param {Node} node Node to test
 * @return {boolean}
 */
function isDocument(node) {
    return node.nodeName === '#document';
}
/**
 * Determines if a given node is a document fragment or not
 * @param {Node} node Node to test
 * @return {boolean}
 */
function isDocumentFragment(node) {
    return node.nodeName === '#document-fragment';
}
/**
 * Determines if a given node is a template node or not
 * @param {Node} node Node to test
 * @return {boolean}
 */
function isTemplateNode(node) {
    return node.nodeName === 'template';
}
const isElementNode = parse5.defaultTreeAdapter.isElementNode;
const isCommentNode = parse5.defaultTreeAdapter.isCommentNode;
const isDocumentTypeNode = parse5.defaultTreeAdapter.isDocumentTypeNode;
const isTextNode = parse5.defaultTreeAdapter.isTextNode;
/**
 * Determines if a given node is a parent or not
 * @param {Node} node Node to test
 * @return {boolean}
 */
function isParentNode(node) {
    return (isDocument(node) ||
        isDocumentFragment(node) ||
        isElementNode(node) ||
        isTemplateNode(node));
}

parse5.defaultTreeAdapter.appendChild;

/**
 * Traverses the tree of a given node
 * @param {Node} node Node to traverse
 * @param {Visitor} visitor Visitor to apply
 * @param {ParentNode=} parent Parent node of the current node
 * @return {void}
 */
function traverse(node, visitor, parent) {
    const shouldVisitChildren = typeof visitor['pre:node'] !== 'function' ||
        visitor['pre:node'](node, parent) !== false;
    if (shouldVisitChildren && isParentNode(node)) {
        for (const child of node.childNodes) {
            traverse(child, visitor, node);
        }
    }
    if (typeof visitor.node === 'function') {
        visitor.node(node, parent);
    }
    if (typeof visitor.document === 'function' && isDocument(node)) {
        visitor.document(node);
    }
    if (typeof visitor.documentFragment === 'function' &&
        isDocumentFragment(node)) {
        visitor.documentFragment(node, parent);
    }
    if (typeof visitor.element === 'function' && isElementNode(node)) {
        visitor.element(node, parent);
    }
    if (typeof visitor.template === 'function' && isTemplateNode(node)) {
        visitor.template(node, parent);
    }
    if (typeof visitor.comment === 'function' && isCommentNode(node)) {
        visitor.comment(node, parent);
    }
    if (typeof visitor.text === 'function' && isTextNode(node)) {
        visitor.text(node, parent);
    }
    if (typeof visitor.documentType === 'function' && isDocumentTypeNode(node)) {
        visitor.documentType(node, parent);
    }
}

function htmlHandler(rawSource, options) {
  const { runtimeSet, classGenerator } = options;
  const fragment = parse5.parse(rawSource);
  traverse(fragment, {
    element(node) {
      const attribute = node.attrs.find((x) => x.name === "class");
      if (attribute) {
        const array = shared.splitCode(attribute.value, {
          splitQuote: false
        });
        for (const v of array) {
          if (runtimeSet.has(v)) {
            attribute.value = attribute.value.replace(shared.makeRegex(v), classGenerator.generateClassName(v).name);
          }
        }
      }
    }
  });
  return parse5.serialize(fragment);
}

const isProd = () => process.env.NODE_ENV === "production";

function handleValue(string_, node, options) {
  const { runtimeSet: set, classGenerator: clsGen, splitQuote = true } = options;
  const array = shared.splitCode(string_, {
    splitQuote
  });
  let rawString = string_;
  for (const v of array) {
    if (set.has(v)) {
      let ignoreFlag = false;
      if (Array.isArray(node.leadingComments)) {
        ignoreFlag = node.leadingComments.findIndex((x) => x.value.includes("tw-mangle") && x.value.includes("ignore")) > -1;
      }
      if (!ignoreFlag) {
        rawString = rawString.replace(shared.makeRegex(v), clsGen.generateClassName(v).name);
      }
    }
  }
  return rawString;
}
function jsHandler(rawSource, options) {
  const result = core.transformSync(rawSource, {
    babelrc: false,
    ast: true,
    plugins: [
      () => {
        return {
          visitor: {
            StringLiteral: {
              enter(p) {
                const n = p.node;
                n.value = handleValue(n.value, n, options);
              }
            },
            TemplateElement: {
              enter(p) {
                const n = p.node;
                n.value.raw = handleValue(n.value.raw, n, options);
              }
            },
            CallExpression: {
              enter(p) {
                const n = p.node;
                if (t__namespace.isIdentifier(n.callee) && n.callee.name === "eval" && t__namespace.isStringLiteral(n.arguments[0])) {
                  const res = jsHandler(n.arguments[0].value, options);
                  if (res.code) {
                    n.arguments[0].value = res.code;
                  }
                }
              }
            }
            // noScope: true
          }
        };
      }
    ],
    minified: options.minified ?? isProd(),
    sourceMaps: false,
    configFile: false
  });
  return result;
}

exports.ClassGenerator = shared.ClassGenerator;
exports.cssHandler = cssHandler;
exports.handleValue = handleValue;
exports.htmlHandler = htmlHandler;
exports.jsHandler = jsHandler;
