import { ClassGenerator } from '@tailwindcss-mangle/shared';
export { ClassGenerator } from '@tailwindcss-mangle/shared';
import { StringLiteral, TemplateElement } from '@babel/types';
import { BabelFileResult } from '@babel/core';

interface IClassGeneratorContextItem {
    name: string;
    usedBy: any[];
}
interface IClassGeneratorOptions {
    reserveClassName?: (string | RegExp)[];
    customGenerate?: (original: string, opts: IClassGeneratorOptions, context: Record<string, any>) => string | undefined;
    log?: boolean;
    exclude?: (string | RegExp)[];
    include?: (string | RegExp)[];
    ignoreClass?: (string | RegExp)[];
    classPrefix?: string;
}
interface IHandlerOptions {
    runtimeSet: Set<string>;
    classGenerator: ClassGenerator;
}
interface IHtmlHandlerOptions extends IHandlerOptions {
}
interface IJsHandlerOptions extends IHandlerOptions {
    splitQuote?: boolean;
    minified?: boolean;
}
interface ICssHandlerOptions extends IHandlerOptions {
    scene?: 'loader' | 'process';
    ignoreVueScoped?: boolean;
}
interface ClassSetOutputOptions {
    filename: string;
    dir?: string;
    type: 'all' | 'partial';
}
interface ClassMapOutputOptions {
    filename: string;
    dir?: string;
}
interface Options {
    classGenerator?: IClassGeneratorOptions;
    exclude?: string[];
    include?: string[];
    classSetOutput?: boolean | ClassSetOutputOptions;
    classMapOutput?: boolean | ClassMapOutputOptions;
}

declare function cssHandler(rawSource: string, options: ICssHandlerOptions): string;

declare function htmlHandler(rawSource: string, options: IHtmlHandlerOptions): string;

declare function handleValue(string_: string, node: StringLiteral | TemplateElement, options: IJsHandlerOptions): string;
declare function jsHandler(rawSource: string, options: IJsHandlerOptions): BabelFileResult;

export { ClassMapOutputOptions, ClassSetOutputOptions, IClassGeneratorContextItem, IClassGeneratorOptions, ICssHandlerOptions, IHandlerOptions, IHtmlHandlerOptions, IJsHandlerOptions, Options, cssHandler, handleValue, htmlHandler, jsHandler };
